.TH SECMPEG 1 "1 August 1993" "Unix"
.SH NAME
secmpeg - confidentiality and integrity-tool for MPEG\-I
.SH SYNOPSIS
.B secmpeg
{\-scedCED} [clevel key ilevel] infile.mpg [outfile.mpg]
.SH DESCRIPTION
.I Secmpeg
is a program based on a rather complex algorythm to ensure
a confidentiality and a integrity service for the video-stream
MPEG\-I.
.PP
.I Secmpeg
accepts the following options listed below:
.PP
.TP 4
.B \-s
Switching on silence mode. No output, except errors, will
be generated. Normally,
.I secmpeg
will at least echo the used time for the coding or decoding
process or, during a integrity-check, echo the result
of it. This option has to be given before every other.
.TP 4
.B \-e
Switching to encoding mode. Encoding means to insert
confidentiality and integrity data into the video-stream
and copying the resulting stream to another given file.
A confidentiality level
.B (clevel)
,a security\-key
.B (key)
,a integrity\-level
.B (ilevel)
,a MPEG\-I inputfile
.B infile.mpg
and a outputfile
.B (outfile.mpg)
have to be given on the command line. The resulting stream
will NOT we playable as an MPEG\-I\-video, because additional
data is inserted (even if no encryption is done).
.PP
.RS 4
The following methos are available for the confidentiality\-level:
.TP 4
.B 0
Only the MPEG\-stream\-headers and the first slice\-header of 
every frame are encrypted. This method is very quick, but not safe.
.TP 4
.B 1
The MPEG\-stream\-headers and ALL slice\-headers are encrypted.
This method is still very quick, but still not safe.
.TP 4
.B 2
Everything like method
.B 1
and the first macroblock after each slice\-header is encrypted.
This method is still quick and looks very safe (all offsets are
encrypted, the locating of relevant data looks rather difficult).
.TP 4
.B 3
Everything like method
.B 2
and the I-frames are encrypted completely. The method is not so quick
anymore, but definitely safe (only intracoded macroblocks are copied
from the original stream, but to find there location looks very
difficult). This method is much quicker than encrypting the complete
stream.
.TP 4
.B 4
Everything like method
.B 3
and all intracoded macroblocks are encrypted as well. The method
could be slower than a complete encryption of the whole stream, but
is the safest one.
.PP
The following methos are available for the integrity\-level:
.TP 4
.B 0
A 16\-bit CRC is performed on all header data.
.TP 4
.B 1
A 16\-bit CRC is performed on all header data and all frames.
.TP 4
.B 2
A 32\-bit CRC is performed on all header data.
.TP 4
.B 3
A 32\-bit CRC is performed on all header data and all frames.
.PP
CRC\-checks are very quick. The suggested mode for integrity is
method
.B 3.
.PP
If a negativ value for the confidentiality\-level or integrity\-level
is given (f.e. -1), no encryption or CRC\-checks are encoded.
.RS -4
.TP 4
.B \-d
Switching to decoding mode. Decoding means to extract the original
MPEG\-I\-stream from a encoded
.I secmpeg
\-stream. If a integrity-method was encoded, a integrity-check is
performed while decoding. If the stream was crypted, the key and
the source filename and the target filename have to be given.
Otherwise only the two filenames have to be given.
.TP 4
.B \-c
A integrity\-check is performed on a with
.I secmpeg
encoded MPEG\-I\-stream. No output file is generated, no service
information is deleted during this option. If the option
.B -s
(silence) is enabled, only errors will be generated, otherwise
a message of success will be shown. If the stream was crypted, a additional
key has to be given.
.TP 4
.B \-E
The complete source file will be crypted with a given key to produce
a target file of the same length. The method is save and slow.
.TP4
.B \-D
The completely encrypted file will be decrypted with the given key.
.TP4
.B \-C
The CRC\-value will be generated over the complete source file and
printed to stdout. If the option
.B -s
(silence) is enabled, only errors will be generated.
.PP
Surely, the last three methods can be used on every kind of stream.
.PP
If a dash (\-) is given instead of filenames (source and\/or target)
.I secmpeg
takes input from
.B stdin
and writes output to
.B stdout.
.PP
.TP4
.B \-v
Prints the version\-number and \-date of the program.
.SH EXAMPLES
To encrypt a MPEG\-I\-stream called
.B in.mpg
to
.B out.mpg
without integrity and with method
.B 2
and the encryptionkey 'password' enter :
.PP
.RS 4
.B secmpeg -e 2 password -1 in.mpg out.mpg
.RS -4
.PP
To encode the same file with a full 32\-bit CRC, but without
encryption, enter:
.PP
.RS 4
.B secmpeg -e -1 3 in.mpg out.mpg
.RS -4
.PP
To do both enter:
.PP
.RS 4
.B secmpeg -e 2 password 3 in.mpg out.mpg
.RS -4
.PP
To check the integrity of the last process in silence mode enter:
.PP
.RS 4
.B secmpeg -s -c password out.mpg
.RS -4
.PP
To decode the last file to
.B orig.mpg
enter:
.PP
.RS4
.B secmpeg -d password out.mpg orig.mpg
.RS -4
.PP
Other possible commands are:
.PP
.RS 4
.B secmpeg -e -2 password 3 - - < in.mpg \| more
.br
.B secmpeg -E password in.mpg out.mpg
.br
.B secmpeg -D password out.mpg orig.mpg
.br
.B cat in.mpg | secmpeg -C -
.RS -4
.SH BUGS
Reading from
.B stdin
and writing to
.B stdout
does not work right for DOS systems.
.SH AUTHORS
.I Secmpeg
was developed and programed by:
.PP
Juergen Meyer
.br
Sonnenallee 50
.br
12045 Berlin GERMANY Access: jm@cs.tu-berlin.de
.PP
Frank Gadegast
.br
Leibnizstr. 30
.br
10625 Berlin GERMANY Access: phade@cs.tu-berlin.de
.PP
Permission to use, copy, modify, and distribute this
software and its documentation for any purpose and without
fee is hereby granted, provided that the archive remains
complete, that this author notice will appear in all copies
and as long as you don't try to make money off it, or pretend
that you wrote it.
